/*
Copyright (C) 2013 NTT DATA Corporation

This program is free software; you can redistribute it and/or
Modify it under the terms of the GNU General Public License
as published by the Free Software Foundation, version 2.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.
 */

package com.clustercontrol.cloud.ui.views.actions;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;

import com.clustercontrol.cloud.commons.util.MessageManager;
import com.clustercontrol.cloud.presenter.ICloudInstance;
import com.clustercontrol.cloud.presenter.RegistNodeRequest;
import com.clustercontrol.cloud.ui.dialogs.RegistNodeDialog;
import com.clustercontrol.util.Messages;


public class RegistNodeAction extends SingleSelectionActionEx {
	public static final String ID = "com.clustercontrol.cloud.ui.views.actions.RegistNodeAction";

	private MessageManager messages = MessageManager.getInstance("messages");
	
	private String msgRegistNodeConfirm = messages.getString("message.confirm_some_action_to_subjects", new Object[]{"word.node", "word.regist"});
	private String msgRegistNodeResult = messages.getString("message.result_some_action_to_something", new Object[]{"word.node", "word.regist"});

	@Override
	protected void invoke(IAction action) throws InvocationException {
		ICloudInstance instance = (ICloudInstance)((IStructuredSelection)this.viewPart.getSite().getSelectionProvider().getSelection()).getFirstElement();

		RegistNodeDialog dialog = new RegistNodeDialog(this.viewPart.getSite().getShell()){};
		if(dialog.open() == Window.OK){
			if (
				MessageDialog.openConfirm(
					null,
					Messages.getString("confirmed"),
					msgRegistNodeConfirm + "\n" + instance.getInstanceId())
				){
				
				RegistNodeRequest request = dialog.getOutput();
				instance.registNode(request);
				
				// 成功報告ダイアログを生成
				MessageDialog.openInformation(
					null,
					Messages.getString("successful"),
					msgRegistNodeResult);
			}
			else {
				return;
			}
		}
	}

	@Override
	protected boolean canEnabled(IStructuredSelection selection) {
		ICloudInstance instance = (ICloudInstance)((IStructuredSelection)this.viewPart.getSite().getSelectionProvider().getSelection()).getFirstElement();
		return ICloudInstance.RegistStateType.unregistered.name().equals(instance.getRegistState());
	}
}